// Defines a custom context attribute that publishes performance
// counters automatically.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
// Ported to beta2 by Richard Birkby (rbirkby@thundermain.com)
// who got tired of waiting for me to do it :-)
//
using System;
using System.Threading;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Contexts;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Activation;
using System.Diagnostics;
using System.Text;

public class MyPerformanceCounter : PerformanceCounter
{
	
	~MyPerformanceCounter() {
        try {
            Console.WriteLine("MyPerfCounter.Finalize");
        }
        catch {
        }
	}	
}

[ AttributeUsage(AttributeTargets.Class) ]
public class PerfCounterAttribute : System.Attribute,
                                    IContextAttribute, 
                                    IContextProperty,
                                    IContributeObjectSink
{
    public string Name {
        get { return(strName); }
    }
    
    private string strName;

    public PerfCounterAttribute( string Name )
    {
        strName = Name + "PerfCounter";

        InstallCounters();
        CreateCounters(Name);

        // Putting the Instances counter on this list
        // adds a reference to it so that it doesn't become
        // a candidate for finalization until we're ready for
        // that to happen.
        //
        DeferFinalizeList.Add(ctrInstances);

        ctrInstances.IncrementBy(1);
        ctrInstancesPerSec.IncrementBy(1);
    }

    static PerfCounterAttribute()
    {
        DeferFinalizeList = new System.Collections.ArrayList();
    }

    ~PerfCounterAttribute()
    {
        try {
            Console.WriteLine("PerfCounterAttribute.Finalize");
        }
        catch {
        }

        ctrInstances.IncrementBy(-1);
        DeferFinalizeList.Remove(ctrInstances);

    }

    private const string strCtrCategory = "Object Perf Stats";

    private void InstallCounters()
    {
        if( !PerformanceCounterCategory.Exists(strCtrCategory) )
        {
            CounterCreationData Ctr1Data = new CounterCreationData();
            CounterCreationData Ctr2Data = new CounterCreationData();
            CounterCreationData Ctr3Data = new CounterCreationData();
            CounterCreationData Ctr4Data = new CounterCreationData();

            Ctr1Data.CounterName = "Calls";
            Ctr1Data.CounterHelp = Ctr1Data.CounterName + " counter help";
            Ctr1Data.CounterType = PerformanceCounterType.NumberOfItems64;

            Ctr2Data.CounterName = "Calls/sec";
            Ctr2Data.CounterHelp = Ctr2Data.CounterName + " counter help";
            Ctr2Data.CounterType = PerformanceCounterType.RateOfCountsPerSecond64;

            Ctr3Data.CounterName = "Instances";
            Ctr3Data.CounterHelp = Ctr3Data.CounterName + " counter help";
            Ctr3Data.CounterType = PerformanceCounterType.NumberOfItems64;

            Ctr4Data.CounterName = "Instances/sec";
            Ctr4Data.CounterHelp = Ctr4Data.CounterName + " counter help";
            Ctr4Data.CounterType = PerformanceCounterType.RateOfCountsPerSecond64;

            CounterCreationData[] CounterData = new CounterCreationData[4];
            CounterData[0] = Ctr1Data;
            CounterData[1] = Ctr2Data;
            CounterData[2] = Ctr3Data;
            CounterData[3] = Ctr4Data;

			
            PerformanceCounterCategory.Create(strCtrCategory, "Help here", new CounterCreationDataCollection(CounterData));

            if( PerformanceCounterCategory.Exists(strCtrCategory) )
            {
                Console.WriteLine("Created counter category successfully.");
            }
            else
            {
                Console.WriteLine("Failed to create counter category");
            }
        }
        // else, counters already installed
    }
    
    static public bool RemoveCounters()
    {
        if( PerformanceCounterCategory.Exists(strCtrCategory) )
        {
            PerformanceCounterCategory.Delete(strCtrCategory);
        }

        return(PerformanceCounterCategory.Exists(strCtrCategory) == false);
    }

    static private System.Collections.ArrayList DeferFinalizeList;

    private PerformanceCounter ctrCalls;
    private PerformanceCounter ctrCallsPerSec;
    private PerformanceCounter ctrInstances;
    private PerformanceCounter ctrInstancesPerSec;

    public PerformanceCounter Instances { get { return(ctrInstances); } }
    public PerformanceCounter Calls { get { return(ctrCalls); } }
    public PerformanceCounter CallsPerSec { get { return(ctrCallsPerSec); } }

    private void CreateCounters( string ClassName )
    {
        StringBuilder InstanceName = new StringBuilder();

        InstanceName.AppendFormat( "{0}, {1}, {2}",
                                   System.Diagnostics.Process.GetCurrentProcess().Id,
                                   System.AppDomain.CurrentDomain.FriendlyName,
                                   ClassName );

        ctrCalls = new PerformanceCounter( );
        ctrCalls.ReadOnly=false;
		ctrCalls.CategoryName = strCtrCategory;
        ctrCalls.MachineName = ".";
        ctrCalls.CounterName = "Calls";
        ctrCalls.InstanceName = InstanceName.ToString();

        ctrCallsPerSec = new PerformanceCounter();
		ctrCallsPerSec.ReadOnly=false;
        ctrCallsPerSec.CategoryName = strCtrCategory;
        ctrCallsPerSec.MachineName = ".";
        ctrCallsPerSec.CounterName = "Calls/sec";
        ctrCallsPerSec.InstanceName = InstanceName.ToString();
    
        ctrInstances = new MyPerformanceCounter();
        ctrInstances.ReadOnly=false;
		ctrInstances.CategoryName = strCtrCategory;
        ctrInstances.MachineName = ".";
        ctrInstances.CounterName = "Instances";
        ctrInstances.InstanceName = InstanceName.ToString();

        ctrInstancesPerSec = new PerformanceCounter();
        ctrInstancesPerSec.ReadOnly=false;
		ctrInstancesPerSec.CategoryName = strCtrCategory;
        ctrInstancesPerSec.MachineName = ".";
        ctrInstancesPerSec.CounterName = "Instances/sec";
        ctrInstancesPerSec.InstanceName = InstanceName.ToString();
    }

    // IContextAttribute
    //
    public bool IsContextOK( Context ctx, IConstructionCallMessage msg )
    {
        return(false);
    }

    public bool IsNewContextOK( Context ctx )
    {
        return(true);
    }

    public void GetPropertiesForNewContext( IConstructionCallMessage msg )
    {
        msg.ContextProperties.Add(this);
    }

    public void Freeze( Context ctx )
    {
    }

    // IContributeObjectSink
    //
    public IMessageSink GetObjectSink( MarshalByRefObject obj, IMessageSink nextSink )
    {
        return(new PerfCounterSink(this, nextSink));
    }
}

class PerfCounterSink : IMessageSink
{
    private IMessageSink _nextSink;
    private PerfCounterAttribute _attr;

    public PerfCounterSink( PerfCounterAttribute attr, IMessageSink nextSink )
    {
        _nextSink = nextSink;
        _attr = attr;
    }

    ~PerfCounterSink()
    {
        try {
            Console.WriteLine("PerfCounterSink.Finalize");
        }
        catch {
        }
    }

    public IMessage SyncProcessMessage(IMessage reqMsg)
    {
        _attr.Calls.IncrementBy(1);
        _attr.CallsPerSec.IncrementBy(1);

        return _nextSink.SyncProcessMessage(reqMsg);
    }

    public IMessageCtrl AsyncProcessMessage(IMessage reqMsg, IMessageSink replySink)
    {
        _attr.Calls.IncrementBy(1);
        _attr.CallsPerSec.IncrementBy(1);

        return _nextSink.AsyncProcessMessage(reqMsg, replySink);
    }

    public IMessageSink NextSink
    {
        get
        {
            return _nextSink;
        }
    }
}

[ PerfCounterAttribute("Foo") ]
class Foo : ContextBoundObject
{
    public void Bar()
    {
        Console.WriteLine("Foo.Bar");
    }

    ~Foo()
    {
        try {
            Console.WriteLine("Foo.Finalize");
        }
        catch {
        }
    }
}

public class Test
{
    static public int Main( System.String[] Args )
    {
        return (new Test())._Main(Args);
    }

    ~Test()
    {
        try {
            Console.WriteLine("Test.Finalize");
        }
        catch {
        }
    }

    void ThreadProc()
    {
        Foo f = new Foo();

        for( long n = 0; ; n++ )
        {
            f.Bar();

            if( (n % 2) == 0 )
            {
                new Foo();
            }

            Thread.Sleep(250);
        }
    }

    int _Main( System.String[] Args )
    {
        if( Args.Length > 0 )
        {
            if( PerfCounterAttribute.RemoveCounters() )
            {
                Console.WriteLine("Removed counters from machine.");
            }
            else
            {
                Console.WriteLine("Failed to remove counters from machine.");
            }

            return(0);
        }

        Thread theThread = new Thread( new ThreadStart(ThreadProc) );
        theThread.Start();

        for( int n = 0; n < 3; n++ )
        {
            Console.WriteLine("Press ENTER to exit.");
            Console.ReadLine();
            System.GC.Collect();
        }

        theThread.Abort();
        theThread.Join();
        return(0);
    }
}
